/** @file
  Copyright (c) 2018 - 2019, Intel Corporation. All rights reserved.<BR>
  Copyright (c) 2021, American Megatrends International LLC.<BR>
  SPDX-License-Identifier: BSD-2-Clause-Patent

**/

/*
 * 
 * Intel ACPI Component Architecture
 * 
 * 
 */
#ifndef __AML_OFFSET_TABLE_H
#define __AML_OFFSET_TABLE_H

typedef struct {
    char                   *Pathname;      /* Full pathname (from root) to the object */
    unsigned short         ParentOpcode;   /* AML opcode for the parent object */
    unsigned long          NamesegOffset;  /* Offset of last nameseg in the parent namepath */
    unsigned char          Opcode;         /* AML opcode for the data */
    unsigned long          Offset;         /* Offset for the data */
    unsigned long long     Value;          /* Original value of the data (as applicable) */
} AML_OFFSET_TABLE_ENTRY;

#endif /* __AML_OFFSET_TABLE_H */

/*
 * Information specific to the supported object types:
 *
 * Integers:
 *    Opcode is the integer prefix, indicates length of the data
 *        (One of: BYTE, WORD, DWORD, QWORD, ZERO, ONE, ONES)
 *    Offset points to the actual integer data
 *    Value is the existing value in the AML
 *
 * Packages:
 *    Opcode is the package or var_package opcode
 *    Offset points to the package opcode
 *    Value is the package element count
 *
 * Operation Regions:
 *    Opcode is the address integer prefix, indicates length of the data
 *    Offset points to the region address
 *    Value is the existing address value in the AML
 *
 * Control Methods:
 *    Offset points to the method flags byte
 *    Value is the existing flags value in the AML
 *
 * Processors:
 *    Offset points to the first byte of the PBlock Address
 *
 * Resource Descriptors:
 *    Opcode is the descriptor type
 *    Offset points to the start of the descriptor
 *
 * Scopes/Devices/ThermalZones:
 *    Nameseg offset only
 */
AML_OFFSET_TABLE_ENTRY   DSDT_PLATWFP__OffsetTable[] =
{
    {"PSYS",                       0x5B80, 0x00000465, 0x0C, 0x0000046B, 0x0000000030584946}, /* OPERATIONREGION */
    {"_SB_.PC00.FIX1",             0x0011, 0x00000000, 0x88, 0x0000D261, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC00.FIX2",             0x0011, 0x00000000, 0x88, 0x0000D289, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC00.FIX5",             0x0011, 0x00000000, 0x87, 0x0000D299, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC00.FIX3",             0x0011, 0x00000000, 0x87, 0x0000D2E7, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC00.FIX4",             0x0011, 0x00000000, 0x8A, 0x0000D301, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC01.FIX1",             0x0011, 0x00000000, 0x88, 0x0000EB75, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC01.FIX5",             0x0011, 0x00000000, 0x87, 0x0000EB85, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC01.FIX2",             0x0011, 0x00000000, 0x88, 0x0000EB9F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC01.FIX6",             0x0011, 0x00000000, 0x88, 0x0000EBAF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC01.FIX7",             0x0011, 0x00000000, 0x88, 0x0000EBBF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC01.FIX3",             0x0011, 0x00000000, 0x87, 0x0000EBCF, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC01.FIX4",             0x0011, 0x00000000, 0x8A, 0x0000EBE9, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC01.BR1A.MCTL",        0x5B80, 0x0000EC6B, 0x0C, 0x0000EC71, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC01.BR1B.MCTL",        0x5B80, 0x0000F48A, 0x0C, 0x0000F490, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC01.BR1C.MCTL",        0x5B80, 0x0000FCA9, 0x0C, 0x0000FCAF, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC01.BR1D.MCTL",        0x5B80, 0x000104C8, 0x0C, 0x000104CE, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC02.FIX1",             0x0011, 0x00000000, 0x88, 0x00010F65, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC02.FIX5",             0x0011, 0x00000000, 0x87, 0x00010F75, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC02.FIX2",             0x0011, 0x00000000, 0x88, 0x00010F8F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC02.FIX6",             0x0011, 0x00000000, 0x88, 0x00010F9F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC02.FIX7",             0x0011, 0x00000000, 0x88, 0x00010FAF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC02.FIX3",             0x0011, 0x00000000, 0x87, 0x00010FBF, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC02.FIX4",             0x0011, 0x00000000, 0x8A, 0x00010FD9, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC02.BR2A.MCTL",        0x5B80, 0x0001105B, 0x0C, 0x00011061, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC02.BR2B.MCTL",        0x5B80, 0x00011A39, 0x0C, 0x00011A3F, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC02.BR2C.MCTL",        0x5B80, 0x00012258, 0x0C, 0x0001225E, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC02.BR2D.MCTL",        0x5B80, 0x00012A77, 0x0C, 0x00012A7D, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC03.FIX1",             0x0011, 0x00000000, 0x88, 0x000134AE, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC03.FIX5",             0x0011, 0x00000000, 0x87, 0x000134BE, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC03.FIX2",             0x0011, 0x00000000, 0x88, 0x000134D8, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC03.FIX6",             0x0011, 0x00000000, 0x88, 0x000134E8, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC03.FIX7",             0x0011, 0x00000000, 0x88, 0x000134F8, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC03.FIX3",             0x0011, 0x00000000, 0x87, 0x00013508, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC03.FIX4",             0x0011, 0x00000000, 0x8A, 0x00013522, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC03.BR3A.MCTL",        0x5B80, 0x000135A4, 0x0C, 0x000135AA, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC03.BR3B.MCTL",        0x5B80, 0x00013DC3, 0x0C, 0x00013DC9, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC03.BR3C.MCTL",        0x5B80, 0x000145E2, 0x0C, 0x000145E8, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC03.BR3D.MCTL",        0x5B80, 0x00014E01, 0x0C, 0x00014E07, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC04.FIX1",             0x0011, 0x00000000, 0x88, 0x000157B2, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC04.FIX5",             0x0011, 0x00000000, 0x87, 0x000157C2, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC04.FIX2",             0x0011, 0x00000000, 0x88, 0x000157DC, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC04.FIX6",             0x0011, 0x00000000, 0x88, 0x000157EC, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC04.FIX7",             0x0011, 0x00000000, 0x88, 0x000157FC, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC04.FIX3",             0x0011, 0x00000000, 0x87, 0x0001580C, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC04.FIX4",             0x0011, 0x00000000, 0x8A, 0x00015826, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC04.MCP0.MCTL",        0x5B80, 0x000158A8, 0x0C, 0x000158AE, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC05.FIX1",             0x0011, 0x00000000, 0x88, 0x000161ED, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC05.FIX5",             0x0011, 0x00000000, 0x87, 0x000161FD, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC05.FIX2",             0x0011, 0x00000000, 0x88, 0x00016217, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC05.FIX6",             0x0011, 0x00000000, 0x88, 0x00016227, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC05.FIX7",             0x0011, 0x00000000, 0x88, 0x00016237, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC05.FIX3",             0x0011, 0x00000000, 0x87, 0x00016247, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC05.FIX4",             0x0011, 0x00000000, 0x8A, 0x00016261, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC05.MCP1.MCTL",        0x5B80, 0x000162E3, 0x0C, 0x000162E9, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC06.FIX1",             0x0011, 0x00000000, 0x88, 0x00017097, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC06.FIX5",             0x0011, 0x00000000, 0x87, 0x000170A7, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC06.FIX2",             0x0011, 0x00000000, 0x88, 0x000170C1, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC06.FIX6",             0x0011, 0x00000000, 0x88, 0x000170D1, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC06.FIX7",             0x0011, 0x00000000, 0x88, 0x000170E1, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC06.FIX3",             0x0011, 0x00000000, 0x87, 0x000170F1, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC06.FIX4",             0x0011, 0x00000000, 0x8A, 0x0001710B, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC06.QRP0.MCTL",        0x5B80, 0x00017207, 0x0C, 0x0001720D, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC07.FIX1",             0x0011, 0x00000000, 0x88, 0x00017C80, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC07.FIX5",             0x0011, 0x00000000, 0x87, 0x00017C90, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC07.FIX2",             0x0011, 0x00000000, 0x88, 0x00017CAA, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC07.FIX6",             0x0011, 0x00000000, 0x88, 0x00017CBA, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC07.FIX7",             0x0011, 0x00000000, 0x88, 0x00017CCA, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC07.FIX3",             0x0011, 0x00000000, 0x87, 0x00017CDA, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC07.FIX4",             0x0011, 0x00000000, 0x8A, 0x00017CF4, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC07.QR1A.MCTL",        0x5B80, 0x00017D86, 0x0C, 0x00017D8C, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC07.QR1B.MCTL",        0x5B80, 0x000185C0, 0x0C, 0x000185C6, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC07.QR1C.MCTL",        0x5B80, 0x00018DFA, 0x0C, 0x00018E00, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC07.QR1D.MCTL",        0x5B80, 0x00019634, 0x0C, 0x0001963A, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC08.FIX1",             0x0011, 0x00000000, 0x88, 0x0001A102, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC08.FIX5",             0x0011, 0x00000000, 0x87, 0x0001A112, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC08.FIX2",             0x0011, 0x00000000, 0x88, 0x0001A12C, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC08.FIX6",             0x0011, 0x00000000, 0x88, 0x0001A13C, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC08.FIX7",             0x0011, 0x00000000, 0x88, 0x0001A14C, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC08.FIX3",             0x0011, 0x00000000, 0x87, 0x0001A15C, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC08.FIX4",             0x0011, 0x00000000, 0x8A, 0x0001A176, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC08.QR2A.MCTL",        0x5B80, 0x0001A208, 0x0C, 0x0001A20E, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC08.QR2B.MCTL",        0x5B80, 0x0001AA42, 0x0C, 0x0001AA48, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC08.QR2C.MCTL",        0x5B80, 0x0001B27C, 0x0C, 0x0001B282, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC08.QR2D.MCTL",        0x5B80, 0x0001BAB6, 0x0C, 0x0001BABC, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC09.FIX1",             0x0011, 0x00000000, 0x88, 0x0001C50D, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC09.FIX5",             0x0011, 0x00000000, 0x87, 0x0001C51D, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC09.FIX2",             0x0011, 0x00000000, 0x88, 0x0001C537, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC09.FIX6",             0x0011, 0x00000000, 0x88, 0x0001C547, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC09.FIX7",             0x0011, 0x00000000, 0x88, 0x0001C557, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC09.FIX3",             0x0011, 0x00000000, 0x87, 0x0001C567, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC09.FIX4",             0x0011, 0x00000000, 0x8A, 0x0001C581, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC09.QR3A.MCTL",        0x5B80, 0x0001C613, 0x0C, 0x0001C619, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC09.QR3B.MCTL",        0x5B80, 0x0001CE4D, 0x0C, 0x0001CE53, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC09.QR3C.MCTL",        0x5B80, 0x0001D687, 0x0C, 0x0001D68D, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC09.QR3D.MCTL",        0x5B80, 0x0001DEC1, 0x0C, 0x0001DEC7, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC10.FIX1",             0x0011, 0x00000000, 0x88, 0x0001E892, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC10.FIX5",             0x0011, 0x00000000, 0x87, 0x0001E8A2, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC10.FIX2",             0x0011, 0x00000000, 0x88, 0x0001E8BC, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC10.FIX6",             0x0011, 0x00000000, 0x88, 0x0001E8CC, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC10.FIX7",             0x0011, 0x00000000, 0x88, 0x0001E8DC, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC10.FIX3",             0x0011, 0x00000000, 0x87, 0x0001E8EC, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC10.FIX4",             0x0011, 0x00000000, 0x8A, 0x0001E906, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC10.MCP2.MCTL",        0x5B80, 0x0001E998, 0x0C, 0x0001E99E, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC11.FIX1",             0x0011, 0x00000000, 0x88, 0x0001F2F2, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC11.FIX5",             0x0011, 0x00000000, 0x87, 0x0001F302, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC11.FIX2",             0x0011, 0x00000000, 0x88, 0x0001F31C, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC11.FIX6",             0x0011, 0x00000000, 0x88, 0x0001F32C, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC11.FIX7",             0x0011, 0x00000000, 0x88, 0x0001F33C, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC11.FIX3",             0x0011, 0x00000000, 0x87, 0x0001F34C, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC11.FIX4",             0x0011, 0x00000000, 0x8A, 0x0001F366, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC11.MCP3.MCTL",        0x5B80, 0x0001F3F8, 0x0C, 0x0001F3FE, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC12.FIX1",             0x0011, 0x00000000, 0x88, 0x000201BC, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC12.FIX5",             0x0011, 0x00000000, 0x87, 0x000201CC, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC12.FIX2",             0x0011, 0x00000000, 0x88, 0x000201E6, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC12.FIX6",             0x0011, 0x00000000, 0x88, 0x000201F6, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC12.FIX7",             0x0011, 0x00000000, 0x88, 0x00020206, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC12.FIX3",             0x0011, 0x00000000, 0x87, 0x00020216, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC12.FIX4",             0x0011, 0x00000000, 0x8A, 0x00020230, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC12.RRP0.MCTL",        0x5B80, 0x0002032C, 0x0C, 0x00020332, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC13.FIX1",             0x0011, 0x00000000, 0x88, 0x00020DA5, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC13.FIX5",             0x0011, 0x00000000, 0x87, 0x00020DB5, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC13.FIX2",             0x0011, 0x00000000, 0x88, 0x00020DCF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC13.FIX6",             0x0011, 0x00000000, 0x88, 0x00020DDF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC13.FIX7",             0x0011, 0x00000000, 0x88, 0x00020DEF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC13.FIX3",             0x0011, 0x00000000, 0x87, 0x00020DFF, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC13.FIX4",             0x0011, 0x00000000, 0x8A, 0x00020E19, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC13.RR1A.MCTL",        0x5B80, 0x00020EAB, 0x0C, 0x00020EB1, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC13.RR1B.MCTL",        0x5B80, 0x000216E5, 0x0C, 0x000216EB, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC13.RR1C.MCTL",        0x5B80, 0x00021F1F, 0x0C, 0x00021F25, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC13.RR1D.MCTL",        0x5B80, 0x00022759, 0x0C, 0x0002275F, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC14.FIX1",             0x0011, 0x00000000, 0x88, 0x00023205, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC14.FIX5",             0x0011, 0x00000000, 0x87, 0x00023215, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC14.FIX2",             0x0011, 0x00000000, 0x88, 0x0002322F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC14.FIX6",             0x0011, 0x00000000, 0x88, 0x0002323F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC14.FIX7",             0x0011, 0x00000000, 0x88, 0x0002324F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC14.FIX3",             0x0011, 0x00000000, 0x87, 0x0002325F, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC14.FIX4",             0x0011, 0x00000000, 0x8A, 0x00023279, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC14.RR2A.MCTL",        0x5B80, 0x0002330B, 0x0C, 0x00023311, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC14.RR2B.MCTL",        0x5B80, 0x00023B45, 0x0C, 0x00023B4B, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC14.RR2C.MCTL",        0x5B80, 0x0002437F, 0x0C, 0x00024385, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC14.RR2D.MCTL",        0x5B80, 0x00024BB9, 0x0C, 0x00024BBF, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC15.FIX1",             0x0011, 0x00000000, 0x88, 0x00025610, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC15.FIX5",             0x0011, 0x00000000, 0x87, 0x00025620, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC15.FIX2",             0x0011, 0x00000000, 0x88, 0x0002563A, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC15.FIX6",             0x0011, 0x00000000, 0x88, 0x0002564A, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC15.FIX7",             0x0011, 0x00000000, 0x88, 0x0002565A, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC15.FIX3",             0x0011, 0x00000000, 0x87, 0x0002566A, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC15.FIX4",             0x0011, 0x00000000, 0x8A, 0x00025684, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC15.RR3A.MCTL",        0x5B80, 0x00025716, 0x0C, 0x0002571C, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC15.RR3B.MCTL",        0x5B80, 0x00025F50, 0x0C, 0x00025F56, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC15.RR3C.MCTL",        0x5B80, 0x0002678A, 0x0C, 0x00026790, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC15.RR3D.MCTL",        0x5B80, 0x00026FC4, 0x0C, 0x00026FCA, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC16.FIX1",             0x0011, 0x00000000, 0x88, 0x00027995, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC16.FIX5",             0x0011, 0x00000000, 0x87, 0x000279A5, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC16.FIX2",             0x0011, 0x00000000, 0x88, 0x000279BF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC16.FIX6",             0x0011, 0x00000000, 0x88, 0x000279CF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC16.FIX7",             0x0011, 0x00000000, 0x88, 0x000279DF, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC16.FIX3",             0x0011, 0x00000000, 0x87, 0x000279EF, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC16.FIX4",             0x0011, 0x00000000, 0x8A, 0x00027A09, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC16.MCP4.MCTL",        0x5B80, 0x00027A9B, 0x0C, 0x00027AA1, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC17.FIX1",             0x0011, 0x00000000, 0x88, 0x000283F5, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC17.FIX5",             0x0011, 0x00000000, 0x87, 0x00028405, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC17.FIX2",             0x0011, 0x00000000, 0x88, 0x0002841F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC17.FIX6",             0x0011, 0x00000000, 0x88, 0x0002842F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC17.FIX7",             0x0011, 0x00000000, 0x88, 0x0002843F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC17.FIX3",             0x0011, 0x00000000, 0x87, 0x0002844F, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC17.FIX4",             0x0011, 0x00000000, 0x8A, 0x00028469, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC17.MCP5.MCTL",        0x5B80, 0x000284FB, 0x0C, 0x00028501, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC18.FIX1",             0x0011, 0x00000000, 0x88, 0x000292BF, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC18.FIX5",             0x0011, 0x00000000, 0x87, 0x000292CF, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC18.FIX2",             0x0011, 0x00000000, 0x88, 0x000292E9, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC18.FIX6",             0x0011, 0x00000000, 0x88, 0x000292F9, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC18.FIX7",             0x0011, 0x00000000, 0x88, 0x00029309, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC18.FIX3",             0x0011, 0x00000000, 0x87, 0x00029319, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC18.FIX4",             0x0011, 0x00000000, 0x8A, 0x00029333, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC18.SRP0.MCTL",        0x5B80, 0x0002942F, 0x0C, 0x00029435, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC19.FIX1",             0x0011, 0x00000000, 0x88, 0x00029EA8, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC19.FIX5",             0x0011, 0x00000000, 0x87, 0x00029EB8, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC19.FIX2",             0x0011, 0x00000000, 0x88, 0x00029ED2, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC19.FIX6",             0x0011, 0x00000000, 0x88, 0x00029EE2, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC19.FIX7",             0x0011, 0x00000000, 0x88, 0x00029EF2, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC19.FIX3",             0x0011, 0x00000000, 0x87, 0x00029F02, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC19.FIX4",             0x0011, 0x00000000, 0x8A, 0x00029F1C, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC19.SR1A.MCTL",        0x5B80, 0x00029FAE, 0x0C, 0x00029FB4, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC19.SR1B.MCTL",        0x5B80, 0x0002A7E8, 0x0C, 0x0002A7EE, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC19.SR1C.MCTL",        0x5B80, 0x0002B022, 0x0C, 0x0002B028, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC19.SR1D.MCTL",        0x5B80, 0x0002B85C, 0x0C, 0x0002B862, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC20.FIX1",             0x0011, 0x00000000, 0x88, 0x0002C32A, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC20.FIX5",             0x0011, 0x00000000, 0x87, 0x0002C33A, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC20.FIX2",             0x0011, 0x00000000, 0x88, 0x0002C354, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC20.FIX6",             0x0011, 0x00000000, 0x88, 0x0002C364, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC20.FIX7",             0x0011, 0x00000000, 0x88, 0x0002C374, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC20.FIX3",             0x0011, 0x00000000, 0x87, 0x0002C384, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC20.FIX4",             0x0011, 0x00000000, 0x8A, 0x0002C39E, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC20.SR2A.MCTL",        0x5B80, 0x0002C430, 0x0C, 0x0002C436, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC20.SR2B.MCTL",        0x5B80, 0x0002CC6A, 0x0C, 0x0002CC70, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC20.SR2C.MCTL",        0x5B80, 0x0002D4A4, 0x0C, 0x0002D4AA, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC20.SR2D.MCTL",        0x5B80, 0x0002DCDE, 0x0C, 0x0002DCE4, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC21.FIX1",             0x0011, 0x00000000, 0x88, 0x0002E735, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC21.FIX5",             0x0011, 0x00000000, 0x87, 0x0002E745, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC21.FIX2",             0x0011, 0x00000000, 0x88, 0x0002E75F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC21.FIX6",             0x0011, 0x00000000, 0x88, 0x0002E76F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC21.FIX7",             0x0011, 0x00000000, 0x88, 0x0002E77F, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC21.FIX3",             0x0011, 0x00000000, 0x87, 0x0002E78F, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC21.FIX4",             0x0011, 0x00000000, 0x8A, 0x0002E7A9, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC21.SR3A.MCTL",        0x5B80, 0x0002E83B, 0x0C, 0x0002E841, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC21.SR3B.MCTL",        0x5B80, 0x0002F075, 0x0C, 0x0002F07B, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC21.SR3C.MCTL",        0x5B80, 0x0002F8AF, 0x0C, 0x0002F8B5, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC21.SR3D.MCTL",        0x5B80, 0x000300E9, 0x0C, 0x000300EF, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC22.FIX1",             0x0011, 0x00000000, 0x88, 0x00030ABA, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC22.FIX5",             0x0011, 0x00000000, 0x87, 0x00030ACA, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC22.FIX2",             0x0011, 0x00000000, 0x88, 0x00030AE4, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC22.FIX6",             0x0011, 0x00000000, 0x88, 0x00030AF4, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC22.FIX7",             0x0011, 0x00000000, 0x88, 0x00030B04, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC22.FIX3",             0x0011, 0x00000000, 0x87, 0x00030B14, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC22.FIX4",             0x0011, 0x00000000, 0x8A, 0x00030B2E, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC22.MCP6.MCTL",        0x5B80, 0x00030BC0, 0x0C, 0x00030BC6, 0x0000000038584946}, /* OPERATIONREGION */
    {"_SB_.PC23.FIX1",             0x0011, 0x00000000, 0x88, 0x0003151A, 0x0000000000000000}, /* WORDBUSNUMBER */
    {"_SB_.PC23.FIX5",             0x0011, 0x00000000, 0x87, 0x0003152A, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC23.FIX2",             0x0011, 0x00000000, 0x88, 0x00031544, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC23.FIX6",             0x0011, 0x00000000, 0x88, 0x00031554, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC23.FIX7",             0x0011, 0x00000000, 0x88, 0x00031564, 0x0000000000000000}, /* WORDIO */
    {"_SB_.PC23.FIX3",             0x0011, 0x00000000, 0x87, 0x00031574, 0x0000000000000000}, /* DWORDMEMORY */
    {"_SB_.PC23.FIX4",             0x0011, 0x00000000, 0x8A, 0x0003158E, 0x0000000000000000}, /* QWORDMEMORY */
    {"_SB_.PC23.MCP7.MCTL",        0x5B80, 0x00031620, 0x0C, 0x00031626, 0x0000000038584946}, /* OPERATIONREGION */
    {NULL,0,0,0,0,0} /* Table terminator */
};

